from django.urls import path

from . import views


app_name = "htmx"
urlpatterns = [
    path("", views.incident_list, name="incident-list"),
    path("<int:pk>/", views.incident_detail, name="incident-detail"),
    path("update/<str:action>/", views.incidents_update, name="incidents-update"),
    path("filter/", views.filter_form, name="incidents-filter"),
]



from django.http import HttpResponse
from django.shortcuts import render
from django.views.decorators.http import require_GET, require_POST
from django_htmx.http import HttpResponseClientRefresh

from argus.auth.utils import save_preference

from argus_htmx.constants import ALLOWED_PAGE_SIZES
from argus_htmx.incidents.views import HtmxHttpRequest


@require_GET
def page_size_names(request: HtmxHttpRequest) -> HttpResponse:
    page_sizes = sorted(ALLOWED_PAGE_SIZES)
    return render(request, "htmx/page_size/_page_size_list.html", {"page_sizes": page_sizes})


@require_POST
def change_page_size(request: HtmxHttpRequest) -> HttpResponse:
    save_preference(request, request.POST, "argus_htmx", "page_size")
    return HttpResponseClientRefresh()


def user_preferences(request) -> HttpResponse:
    """Renders the main preferences page for a user"""
    context = {
        "page_title": "User preferences",
    }
    return render(request, "htmx/user/preferences.html", context=context)



from __future__ import annotations

import logging
from datetime import datetime

from django import forms
from django.contrib.auth import get_user_model
from django.shortcuts import render, get_object_or_404

from django.views.decorators.http import require_POST, require_GET
from django.core.paginator import Paginator
from django.http import HttpRequest, HttpResponse, HttpResponseBadRequest
from django_htmx.middleware import HtmxDetails
from django_htmx.http import HttpResponseClientRefresh

from argus.auth.utils import get_preference, save_preference
from argus.incident.models import Incident
from argus.util.datetime_utils import make_aware

from .constants import ALLOWED_PAGE_SIZES
from .customization import get_incident_table_columns
from .utils import get_filter_function
from .forms import AckForm, DescriptionOptionalForm, EditTicketUrlForm, AddTicketUrlForm
from ..utils import (
    bulk_change_incidents,
    bulk_ack_queryset,
    bulk_close_queryset,
    bulk_reopen_queryset,
    bulk_change_ticket_url_queryset,
)

User = get_user_model()
LOG = logging.getLogger(__name__)


# Map request trigger to parameters for incidents update
INCIDENT_UPDATE_ACTIONS = {
    "ack": (AckForm, bulk_ack_queryset),
    "close": (DescriptionOptionalForm, bulk_close_queryset),
    "reopen": (DescriptionOptionalForm, bulk_reopen_queryset),
    "update-ticket": (EditTicketUrlForm, bulk_change_ticket_url_queryset),
    "add-ticket": (AddTicketUrlForm, bulk_change_ticket_url_queryset),
}


def prefetch_incident_daughters():
    return Incident.objects.select_related("source").prefetch_related(
        "incident_tag_relations",
        "incident_tag_relations__tag",
        "events",
        "events__ack",
    )


class HtmxHttpRequest(HttpRequest):
    htmx: HtmxDetails


# fetch with htmx
def incident_detail(request, pk: int):
    incident = get_object_or_404(Incident, id=pk)
    context = {
        "incident": incident,
        "page_title": str(incident),
    }
    return render(request, "htmx/incidents/incident_detail.html", context=context)


def get_form_data(request, formclass: forms.Form):
    formdata = request.POST or None
    incident_ids = []
    cleaned_form = None
    if formdata:
        incident_ids = request.POST.getlist("incident_ids", [])
        form = formclass(formdata)
        if form.is_valid():
            cleaned_form = form.cleaned_data
    return cleaned_form, incident_ids


@require_POST
def incidents_update(request: HtmxHttpRequest, action: str):
    try:
        formclass, callback_func = INCIDENT_UPDATE_ACTIONS[action]
    except KeyError:
        LOG.error("Unrecognized action name %s when updating incidents.", action)
        return HttpResponseBadRequest("Invalid update action")
    formdata, incident_ids = get_form_data(request, formclass)
    if formdata:
        bulk_change_incidents(request.user, incident_ids, formdata, callback_func)
    return HttpResponseClientRefresh()


@require_GET
def filter_form(request: HtmxHttpRequest):
    incident_list_filter = get_filter_function()
    filter_form, _ = incident_list_filter(request, None)
    context = {"filter_form": filter_form}
    return render(request, "htmx/incidents/_incident_filterbox.html", context=context)


def incident_list(request: HtmxHttpRequest) -> HttpResponse:
    columns = get_incident_table_columns()

    # Load incidents
    qs = prefetch_incident_daughters().order_by("-start_time")
    total_count = qs.count()
    last_refreshed = make_aware(datetime.now())

    params = dict(request.GET.items())

    incident_list_filter = get_filter_function()
    filter_form, qs = incident_list_filter(request, qs)
    filtered_count = qs.count()

    # Standard Django pagination

    page_size = get_preference(request, "argus_htmx", "page_size")
    success = save_preference(request, request.GET, "argus_htmx", "page_size")
    if success:
        page_size = get_preference(request, "argus_htmx", "page_size")
    paginator = Paginator(object_list=qs, per_page=page_size)
    page_num = params.pop("page", "1")
    page = paginator.get_page(page_num)

    # The htmx magic - use a different, minimal base template for htmx
    # requests, allowing us to skip rendering the unchanging parts of the
    # template.
    if request.htmx:
        base_template = "htmx/incidents/responses/_incidents_table_refresh.html"
    else:
        base_template = "htmx/incidents/_base.html"
    context = {
        "columns": columns,
        "filtered_count": filtered_count,
        "count": total_count,
        "filter_form": filter_form,
        "page_title": "Incidents",
        "base": base_template,
        "page": page,
        "last_refreshed": last_refreshed,
        "update_interval": 30,
        "all_page_sizes": ALLOWED_PAGE_SIZES,
    }

    return render(request, "htmx/incidents/incident_list.html", context=context)



from django import template


register = template.Library()


@register.filter
def tagvalues(incident, key) -> list:
    """Return values of tags with key KEY for incident INCIDENT

    There can be multiple tags with the same key
    """
    tags = incident.deprecated_tags
    return [str(tag.value) for tag in tags if tag.key == key]


@register.filter
def is_acked_by(incident, group: str) -> bool:
    return incident.is_acked_by(group)


@register.filter
def pp_level(level: int) -> str:
    level = str(level)
    mapping = {
        "1": "Critical",
        "2": "High",
        "3": "Moderate",
        "4": "Low",
        "5": "Information",
    }
    if level not in mapping:
        return mapping["5"]
    return mapping[level]



"""
How to use:

Append the "context_processors" list for the TEMPLATES-backend
``django.template.backends.django.DjangoTemplates`` with the full dotted path.

See django settings for ``TEMPLATES``.
"""

from . import settings as fallback
from django.conf import settings


def path_to_stylesheet(request):
    return {"path_to_stylesheet": getattr(settings, "STYLESHEET_PATH", fallback.STYLESHEET_PATH)}



from django.conf import settings
from django.contrib.auth.views import redirect_to_login
from django.http import HttpRequest, HttpResponse
from django.template import loader
from django.utils.deprecation import MiddlewareMixin
from django.utils.encoding import force_str


class LoginRequiredMiddleware:
    def __init__(self, get_response):
        self.public_urls = getattr(settings, "PUBLIC_URLS", ())
        self.login_url = force_str(settings.LOGIN_URL)
        self.get_response = get_response

    def __call__(self, request):
        response = self.get_response(request)
        return response

    def process_view(self, request, view_func, _view_args, _view_kwargs):
        assert hasattr(request, "user"), (
            "The LoginRequiredMiddleware requires authentication middleware "
            "to be installed. Edit your MIDDLEWARE%s setting to insert "
            "'django.contrib.auth.middleware.AuthenticationMiddleware' "
            "before this middleware." % ("_CLASSES" if settings.MIDDLEWARE is None else "")
        )

        # If path is public, allow
        for url in self.public_urls:
            if request.path.startswith(url):
                return None

        # If CBV has the attribute login_required == False, allow
        view_class = getattr(view_func, "view_class", None)
        if view_class and not getattr(view_class, "login_required", True):
            return None

        # If view_func.login_required == False, allow
        if not getattr(view_func, "login_required", True):
            return None

        # Allow authenticated users
        if request.user.is_authenticated:
            return None

        # Redirect unauthenticated users to login page
        return redirect_to_login(request.get_full_path(), self.login_url, "next")


class HtmxMessageMiddleware(MiddlewareMixin):
    """
    For htmx requests, adds messages to the #notification-messages div defined in
    `templates/messages/_notification_messages.html` using htmx's hx-swap-oob feature
    """

    TEMPLATE = "messages/_notification_messages_htmx_append.html"

    def process_response(self, request: HttpRequest, response: HttpResponse) -> HttpResponse:
        if not request.htmx:
            return response

        # Ignore redirections because HTMX cannot read the headers
        if 300 <= response.status_code < 400:
            return response

        if not response.writable():
            return response

        response.write(loader.render_to_string(self.TEMPLATE, request=request))
        return response



# customize the displayed columns in the incident table
# items in INCIDENT_TABLE_COLUMNS can be either a `str` referring to a key in
# argus_htmx.incidents.customization.BUILTIN_COLUMNS or an instance of
# argus_htmx.incidents.customization.IncidentTableColumn
from argus.site.settings import get_json_env, get_str_env

INCIDENT_TABLE_COLUMNS = [
    "row_select",
    "id",
    "start_time",
    "combined_status",
    "level",
    "source",
    "description",
    "ticket",
]

# These templates are auto-discovered by the templating engine and are relative
# to the argus_htmx's `templates/` directory
TAILWIND_CONFIG_TEMPLATE = "tailwind/tailwind.config.js"
TAILWIND_CSS_TEMPLATE = "tailwind/styles.css"

# These targets are relative to the working directory when running the `tailwind_config`
# management command, which should be done from the repostitory root
TAILWIND_CONFIG_TARGET = "src/argus_htmx/tailwindtheme/tailwind.config.js"
TAILWIND_CSS_TARGET = "src/argus_htmx/tailwindtheme/styles.css"

STYLESHEET_PATH_DEFAULT = "styles.css"
DEFAULT_THEMES = [
    "dark",
    "light",
    {
        "argus": {
            "primary": "#006d91",
            "primary-content": "#d1e1e9",
            "secondary": "#f3b61f",
            "secondary-content": "#140c00",
            "accent": "#c84700",
            "accent-content": "#f8dbd1",
            "neutral": "#006d91",
            "neutral-content": "#d1e1e9",
            "base-100": "#edfaff",
            "base-200": "#ced9de",
            "base-300": "#b0babd",
            "base-content": "#141516",
            "info": "#0073e5",
            "info-content": "#000512",
            "success": "#008700",
            "success-content": "#d3e7d1",
            "warning": "#ee4900",
            "warning-content": "#140200",
            "error": "#e5545a",
            "error-content": "#120203",
        }
    },
]
DAISYUI_THEMES = get_json_env("DAISYUI_THEMES", DEFAULT_THEMES, quiet=True)
THEME_DEFAULT = get_str_env("ARGUS_THEME_DEFAULT", "argus")
DEFAULT_THEME_OVERRIDE = {}
TAILWIND_THEME_OVERRIDE = get_json_env("TAILWIND_THEME_OVERRIDE", DEFAULT_THEME_OVERRIDE, quiet=True)
STYLESHEET_PATH = get_str_env("ARGUS_STYLESHEET_PATH", STYLESHEET_PATH_DEFAULT)



from django.conf import settings


__all__ = [
    "DEFAULT_PAGE_SIZE",
    "ALLOWED_PAGE_SIZES",
    "PAGE_SIZE_CHOICES",
]


DEFAULT_PAGE_SIZE = getattr(settings, "ARGUS_INCIDENTS_DEFAULT_PAGE_SIZE", 10)
ALLOWED_PAGE_SIZES = getattr(settings, "ARGUS_INCIDENTS_PAGE_SIZES", [10, 20, 50, 100])
PAGE_SIZE_CHOICES = tuple((ps, ps) for ps in ALLOWED_PAGE_SIZES)



from argus.site.settings._serializers import ListAppSetting


__all__ = ["APP_SETTINGS"]

_app_settings = [
    {
        "settings": {
            "LOGIN_URL": "/accounts/login/",
            "LOGOUT_URL": "/accounts/logout/",
            "LOGIN_REDIRECT_URL": "/incidents/",
            "LOGOUT_REDIRECT_URL": "/incidents/",
            "PUBLIC_URLS": [
                "/accounts/login/",
                "/api/",
                "/oidc/",
            ],
        },
    },
    {
        "app_name": "argus_htmx",
        "urls": {
            "path": "",
            "urlpatterns_module": "argus_htmx.urls",
        },
        "context_processors": [
            "argus.auth.context_processors.preferences",
            "argus_htmx.context_processors.path_to_stylesheet",
        ],
        "middleware": {
            "argus_htmx.middleware.LoginRequiredMiddleware": "end",
            "argus_htmx.middleware.HtmxMessageMiddleware": "end",
        },
    },
    {
        "app_name": "django_htmx",
        "middleware": {
            "django_htmx.middleware.HtmxMiddleware": "end",
        },
    },
    {
        "app_name": "widget_tweaks",  # indent for readability
    },
]

APP_SETTINGS = ListAppSetting(_app_settings).root



from django.conf import settings
from django.contrib.auth.views import LoginView as DjangoLoginView
from django.urls import reverse

from argus.auth.utils import (
    has_model_backend,
    has_remote_user_backend,
    get_psa_authentication_backends,
    get_authentication_backend_classes,
)


REMOTE_USER_METHOD_NAME = getattr(settings, "ARGUS_REMOTE_USER_METHOD_NAME", "REMOTE_USER")
OIDC_METHOD_NAME = getattr(settings, "ARGUS_OIDC_METHOD_NAME", "OIDC")


def get_htmx_authentication_backend_name_and_type():
    # Needed for HTMX LoginView
    backends = get_authentication_backend_classes()

    data = {}
    if has_model_backend(backends):
        data["local"] = {
            "url": reverse("htmx:login"),
            "display_name": "Log In",
        }

    if has_remote_user_backend(backends):
        remote_user_data = {
            "url": "/",  # Should probably also be a setting
            "display_name": REMOTE_USER_METHOD_NAME,
        }
        data.setdefault("external", []).append(remote_user_data)

    for backend in get_psa_authentication_backends(backends):
        display_name = backend.name
        if backend.name == "oidc":
            display_name = OIDC_METHOD_NAME
        psa_backend_data = {
            "url": reverse("social:begin", kwargs={"backend": backend.name}),
            "display_name": display_name,
        }
        data.setdefault("external", []).append(psa_backend_data)

    return data


class LoginView(DjangoLoginView):
    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        backends = get_htmx_authentication_backend_name_and_type()
        context["backends"] = backends
        return context



from django import forms

from argus.auth.models import preferences

from argus_htmx.constants import (
    DATETIME_FORMATS,
    DATETIME_DEFAULT,
    DATETIME_CHOICES,
    PAGE_SIZE_CHOICES,
    DEFAULT_PAGE_SIZE,
    THEME_CHOICES,
    THEME_DEFAULT,
)


class DateTimeFormatForm(forms.Form):
    datetime_format_name = forms.ChoiceField(required=False, choices=DATETIME_CHOICES)


class PageSizeForm(forms.Form):
    page_size = forms.TypedChoiceField(required=False, choices=PAGE_SIZE_CHOICES, coerce=int)


class ThemeForm(forms.Form):
    theme = forms.ChoiceField(choices=THEME_CHOICES)


@preferences(namespace="argus_htmx")
class ArgusHtmxPreferences:
    FORMS = {
        "datetime_format_name": DateTimeFormatForm,
        "page_size": PageSizeForm,
        "theme": ThemeForm,
    }
    _FIELD_DEFAULTS = {
        "datetime_format_name": DATETIME_DEFAULT,
        "page_size": DEFAULT_PAGE_SIZE,
        "theme": THEME_DEFAULT,
    }

    def update_context(self, context):
        datetime_format_name = context.get("datetime_format_name", DATETIME_DEFAULT)
        datetime_format = DATETIME_FORMATS[datetime_format_name]
        return {
            "datetime_format": datetime_format.datetime,
            "date_format": datetime_format.date,
            "time_format": datetime_format.time,
        }



import logging
from pathlib import Path
from re import findall

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.contrib.staticfiles.finders import find

from argus_htmx import settings as fallbacks


__all__ = [
    "get_raw_themes_setting",
    "get_theme_names",
    "get_theme_default",
]


LOG = logging.getLogger(__name__)


def get_raw_themes_setting():
    return getattr(settings, "DAISYUI_THEMES", fallbacks.DAISYUI_THEMES)


def get_themes_from_setting():
    themes_setting = get_raw_themes_setting()
    theme_names = []
    for theme in themes_setting:
        if isinstance(theme, str):
            theme_names.append(theme)
        elif isinstance(theme, dict):
            theme_names.extend(theme.keys())
    return theme_names


def get_stylesheet_path():
    return getattr(settings, "STYLESHEET_PATH", fallbacks.STYLESHEET_PATH)


def get_themes_from_css():
    THEME_NAME_RE = "(?P<theme>[-_\w]+)"
    DATA_THEME_RE = f"\[data-theme={THEME_NAME_RE}\]"

    absolute_stylesheet_path = Path(find(get_stylesheet_path()))
    styles_css = absolute_stylesheet_path.read_text()

    return findall(DATA_THEME_RE, styles_css)


def get_theme_names(quiet=True):
    ERROR_MSG = "Themes in settings are out of sync with themes installed"

    themes_from_setting = set(get_themes_from_setting())
    themes_from_css = set(get_themes_from_css())
    installed_themes = themes_from_setting & themes_from_css

    all_themes = themes_from_setting | themes_from_css
    if all_themes != installed_themes:
        LOG.warning(ERROR_MSG)
        if not quiet:
            raise ImproperlyConfigured(ERROR_MSG)

    return installed_themes


def get_theme_default():
    return getattr(settings, "THEME_DEFAULT", fallbacks.THEME_DEFAULT)



from argus_htmx.dateformat.constants import DATETIME_DEFAULT, DATETIME_FORMATS, DATETIME_CHOICES
from argus_htmx.incidents.constants import DEFAULT_PAGE_SIZE, ALLOWED_PAGE_SIZES, PAGE_SIZE_CHOICES
from argus_htmx.themes.constants import THEME_CHOICES, THEME_NAMES, THEME_DEFAULT


__all__ = [
    "ALLOWED_PAGE_SIZES",
    "DATETIME_CHOICES",
    "DATETIME_DEFAULT",
    "DATETIME_FORMATS",
    "DEFAULT_PAGE_SIZE",
    "PAGE_SIZE_CHOICES",
    "THEME_CHOICES",
    "THEME_DEFAULT",
    "THEME_NAMES",
]



import logging

from django.shortcuts import render

from django.views.decorators.http import require_GET, require_POST
from django.http import HttpResponse
from django_htmx.http import HttpResponseClientRefresh

from argus.auth.utils import save_preference

from argus_htmx.constants import THEME_NAMES
from argus_htmx.incidents.views import HtmxHttpRequest

LOG = logging.getLogger(__name__)
THEMES_MODULE = "argus_htmx"


@require_GET
def theme_names(request: HtmxHttpRequest) -> HttpResponse:
    themes = THEME_NAMES
    return render(request, "htmx/themes/_theme_list.html", {"theme_list": themes})


@require_POST
def change_theme(request: HtmxHttpRequest) -> HttpResponse:
    success = save_preference(request, request.POST, "argus_htmx", "theme")
    if success:
        return render(request, "htmx/themes/_current_theme.html")
    return HttpResponseClientRefresh()



from django.urls import path

from . import views


app_name = "htmx"
urlpatterns = [
    path("", views.user_preferences, name="user-preferences"),
    path("page_size/names/", views.page_size_names, name="page-sizes"),
    path("page_size/change/", views.change_page_size, name="change-page-size"),
]



# Should be in argus proper and merged with api bulk actions
#
# THIS IS LESS DRY-ABLE THAN YOU (or SonarCloud) THINK(S)
#
# Uses composition, see bulk_change_incidents()

from typing import List, Dict, Any

from django.utils import timezone

from argus.incident.models import Incident
from argus.util.signals import bulk_changed


def send_changed_incidents(incidents):
    bulk_changed.send(sender=Incident, instances=incidents)


def get_qs_for_incident_ids(incident_ids: List[int], qs=None):
    # setup
    if qs is None:
        qs = Incident.objects.all()
    qs = qs.filter(pk__in=incident_ids)
    found_ids = set(qs.values_list("id", flat=True))

    # wash ids
    missing_ids = set(incident_ids) - found_ids
    return qs, missing_ids


def bulk_ack_queryset(actor, qs, data: Dict[str, Any]):
    timestamp = data["timestamp"]
    description = data.get("description", "")
    expiration = data.get("expiration", None)
    acks = qs.create_acks(actor, timestamp, description, expiration)
    incidents = []
    for ack in acks:
        incidents.append(ack.event.incident)
    return incidents


def bulk_close_queryset(actor, qs, data: Dict[str, Any]):
    timestamp = data["timestamp"]
    description = data.get("description", "")
    events = qs.close(actor, timestamp, description)
    incidents = []
    for event in events:
        incidents.append(event.incident)
    return incidents


def bulk_reopen_queryset(actor, qs, data: Dict[str, Any]):
    timestamp = data["timestamp"]
    description = data.get("description", "")
    events = qs.reopen(actor, timestamp, description)
    incidents = []
    for event in events:
        incidents.append(event.incident)
    return incidents


def bulk_change_ticket_url_queryset(actor, qs, data: Dict[str, Any]):
    timestamp = data["timestamp"]
    ticket_url = data.get("ticket_url", "")
    return qs.update_ticket_url(actor, ticket_url, timestamp=timestamp)


def bulk_change_incidents(actor, incident_ids: List[int], data: Dict[str, Any], func, qs=None):
    """
    Update incidents in bulk

    Applies ``func`` to the incidents in ``incident_ids`` with the
    pre-validated bulk-type dependent key-value pairs in ``data``. Blames it on
    the user ``actor``. Adds ``timestamp`` to ``data`` if it is not already present.

    Returns a queryset of the changed incidents and a list of incident ids that
    could not be changed due to for instance

    - The incidents have been deleted in the meantime
    - We're working on a subset of incidents and the ids are not in that subset
    """
    qs, missing_ids = get_qs_for_incident_ids(incident_ids, qs)
    if not data.get("timestamp"):
        data["timestamp"] = timezone.now()
    incidents = func(actor, qs, data)
    send_changed_incidents(incidents)
    return incidents, missing_ids



"""
Definitions and defaults for UI customization on the incidents page.

Currently customizable UI elements:
- table columns: configure what columns to show in the incidents listing
"""

from dataclasses import dataclass
from typing import List, Optional, Union

from django.conf import settings

from argus_htmx import settings as argus_htmx_settings


@dataclass
class IncidentTableColumn:
    """Class for defining a column in the incidents table.

    :param name: identifier for this column
    :param label: what to show as the column header
    :param cell_template: template to use when rendering a cell for this column
    :param context: additional context to pass to the rendering cell. Will be made
        available as ``cell_context`` in the cell template
    """

    name: str  # identifier
    label: str  # display value
    cell_template: str
    header_template: Optional[str] = None
    context: Optional[dict] = None


_BUILTIN_COLUMN_LIST = [
    IncidentTableColumn(
        "row_select",
        "Selected",
        "htmx/incidents/_incident_checkbox.html",
        "htmx/incidents/_selected_incidents_header.html",
    ),
    IncidentTableColumn("id", "ID", "htmx/incidents/_incident_pk.html"),
    IncidentTableColumn(
        "start_time",
        "Timestamp",
        "htmx/incidents/_incident_start_time.html",
        "htmx/incidents/_incident_start_time_header.html",
    ),
    IncidentTableColumn("status", "Status", "htmx/incidents/_incident_status.html"),
    IncidentTableColumn("level", "Severity level", "htmx/incidents/_incident_level.html"),
    IncidentTableColumn("source", "Source", "htmx/incidents/_incident_source.html"),
    IncidentTableColumn("description", "Description", "htmx/incidents/_incident_description.html"),
    IncidentTableColumn("ack", "Ack", "htmx/incidents/_incident_ack.html"),
    IncidentTableColumn("combined_status", "Status", "htmx/incidents/_incident_combined_status.html"),
    IncidentTableColumn("ticket", "Ticket", "htmx/incidents/_incident_ticket.html"),
    IncidentTableColumn("links", "Actions", "htmx/incidents/_incident_actions.html"),
]
BUILTIN_COLUMNS = {col.name: col for col in _BUILTIN_COLUMN_LIST}


def get_incident_table_columns() -> List[IncidentTableColumn]:
    columns = getattr(settings, "INCIDENT_TABLE_COLUMNS", argus_htmx_settings.INCIDENT_TABLE_COLUMNS)
    return [_resolve_column(col) for col in columns]


def _resolve_column(col: Union[str, IncidentTableColumn]):
    if isinstance(col, str):
        try:
            col = BUILTIN_COLUMNS[col]
        except KeyError:
            raise ValueError(f"Column '{col}' is not defined")
    return col



from django.contrib.auth import views as django_auth_views
from django.urls import path, include

from .auth import views as auth_views
from .incidents.urls import urlpatterns as incident_urls
from .timeslots.urls import urlpatterns as timeslot_urls
from .notificationprofiles.urls import urlpatterns as notificationprofile_urls
from .destinations.urls import urlpatterns as destination_urls
from .themes.urls import urlpatterns as theme_urls
from .dateformat.urls import urlpatterns as dateformat_urls
from .user.urls import urlpatterns as user_urls

app_name = "htmx"
urlpatterns = [
    path("accounts/login/", auth_views.LoginView.as_view(), name="login"),
    path("accounts/logout/", django_auth_views.LogoutView.as_view(), name="logout"),
    # path("accounts/", include("django.contrib.auth.urls")),
    path("incidents/", include(incident_urls)),
    path("timeslots/", include(timeslot_urls)),
    path("notificationprofiles/", include(notificationprofile_urls)),
    path("destinations/", include(destination_urls)),
    path("themes/", include(theme_urls)),
    path("dateformat/", include(dateformat_urls)),
    path("user/", include(user_urls)),
]



# Generated by Django 4.2.16 on 2024-11-07 11:02

from django.db import migrations


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        ("argus_auth", "0004_add_preferences_model"),
    ]

    operations = [
        migrations.CreateModel(
            name="ArgusHtmxPreferences",
            fields=[],
            options={
                "proxy": True,
                "indexes": [],
                "constraints": [],
            },
            bases=("argus_auth.preferences",),
        ),
    ]



import dataclasses
from django.conf import settings

__all__ = [
    "DATETIME_FALLBACK",
    "DATETIME_FORMATS",
    "DATETIME_DEFAULT",
    "DATETIME_CHOICES",
]


@dataclasses.dataclass
class DateTimeFormat:
    datetime: str
    date: str
    time: str


DATETIME_FALLBACK = "LOCALE"

# Datetime format specification can be found here:
# https://docs.djangoproject.com/en/5.1/ref/templates/builtins/#date
DATETIME_FORMATS = {
    "LOCALE": DateTimeFormat(datetime="DATETIME_FORMAT", date="DATE_FORMAT", time="TIME_FORMAT"),  # default
    "ISO": DateTimeFormat(datetime="Y-m-d H:i:s", date="Y-m-d", time="H:i:s"),
    "RFC5322": DateTimeFormat("r", "r", "r"),
    "EPOCH": DateTimeFormat("U", "U", "U"),
}
DATETIME_CHOICES = tuple((format, format) for format in DATETIME_FORMATS)

_datetime_setting = getattr(settings, "ARGUS_FRONTEND_DATETIME_FORMAT", DATETIME_FALLBACK)

DATETIME_DEFAULT = DATETIME_FALLBACK
if _datetime_setting in DATETIME_FORMATS:
    DATETIME_DEFAULT = _datetime_setting



from argus_htmx.themes.utils import get_theme_default, get_theme_names


__all__ = [
    "THEME_CHOICES",
    "THEME_DEFAULT",
    "THEME_NAMES",
]


THEME_NAMES = sorted(get_theme_names())
THEME_CHOICES = tuple((theme, theme) for theme in THEME_NAMES)
THEME_DEFAULT = get_theme_default()



from django.core.checks import Error, Warning, register
from django.core.exceptions import ImproperlyConfigured

from .themes.utils import get_theme_names, get_stylesheet_path


@register
def check_for_valid_themes_list(app_configs, **kwargs):
    styles_path = get_stylesheet_path()
    themes = []
    try:
        themes = get_theme_names(quiet=False)
    except ImproperlyConfigured as e:
        return [
            Warning(
                str(e),
                hint=f"Regenerate {styles_path}",
                id="argus_htmx.T001",
            )
        ]
    if not themes:
        return [
            Error(
                "no themes installed",
                hint=f'Check the settings "DAISYUI_THEMES" and "TAILWIND_THEME_OVERRIDE" and regenerate {styles_path}',
                id="argus_htmx.T002",
            )
        ]
    return []



import logging

from django.shortcuts import render

from django.views.decorators.http import require_GET, require_POST
from django.http import HttpResponse
from django_htmx.http import HttpResponseClientRefresh

from argus.auth.utils import save_preference

from argus_htmx.incidents.views import HtmxHttpRequest
from .constants import DATETIME_FORMATS

LOG = logging.getLogger(__name__)


@require_GET
def dateformat_names(request: HtmxHttpRequest) -> HttpResponse:
    datetime_formats = DATETIME_FORMATS.keys()
    return render(request, "htmx/dateformat/_dateformat_list.html", {"datetime_formats": datetime_formats})


@require_POST
def change_dateformat(request: HtmxHttpRequest) -> HttpResponse:
    save_preference(request, request.POST, "argus_htmx", "datetime_format_name")
    return HttpResponseClientRefresh()
